Bridge

Struct Bridge 

pub struct Bridge<Next, Format>
where Next: Layer, Format: FfiFormat,
{ /* private fields */ }
FFI Bridge with support for wrapping a middleware stack

Implementations

Source

impl<Next, Format> Bridge<Next, Format>
where Next: Layer, Next::Event: for<'a> Deserialize<'a>, Next::Effect: EffectFFI, Format: FfiFormat,

pub fn new<F>(next: Next, effect_callback: F) -> Self
where F: Fn(Result<Vec<u8>, BridgeError<Format>>) + Send + Sync + 'static,

Typically, you would would use Layer::bridge to construct a Bridge instance

pub fn update( &self, event_bytes: &[u8], requests_out: &mut Vec<u8>, ) -> Result<(), BridgeError<Format>>

Send a serialized event to the core

Errors

Returns an BridgeError when any of the (de)serialization fails

pub fn resolve( &self, id: EffectId, output: &[u8], requests_out: &mut Vec<u8>, ) -> Result<(), BridgeError<Format>>

Resolve a requested effect, providing the output to the core

Errors

Returns a BridgeError when the effect fails to resolve, or any of the (de)serialization fails.

pub fn view(&self, view_out: &mut Vec<u8>) -> Result<(), BridgeError<Format>>
where Next::ViewModel: Serialize,

Get the latest view model

Errors

Returns an BridgeError when any of the (de)serialization fails