HandleEffectLayer

Struct HandleEffectLayer 

pub struct HandleEffectLayer<Next, EM>
where Next: Layer + Sync + Send + 'static, Next::Effect: TryInto<Request<EM::Op>, Error = Next::Effect>, EM: EffectMiddleware,
{ /* private fields */ }
Middleware layer able to process some of the effects. This implements the general behaviour making sure all follow-up effects are processed and routed to the right place and delegates to the generic parameter EM, which implements EffectMiddleware.

Implementations

Source

impl<Next, EM> HandleEffectLayer<Next, EM>
where Next: Layer, Next::Effect: TryInto<Request<EM::Op>, Error = Next::Effect>, EM: EffectMiddleware + 'static,

pub fn new(next: Next, middleware: EM) -> Self

Typically, you would use Layer::handle_effects_using to construct a HandleEffectLayer instance for a specific EffectMiddleware.