MapEffectLayer

Struct MapEffectLayer 

pub struct MapEffectLayer<Next, Effect>
where Next: Layer, Effect: 'static,
{ /* private fields */ }
Middleware for converting the effect type to another type.

Typically, this is used to eliminate some of the effect variants which are processed by the layers below, so that code using this stack is not forced to have extraneous match arms which are never called.

Implementations

Source

impl<Next, Effect> MapEffectLayer<Next, Effect>
where Next: Layer,

pub fn new(next: Next) -> Self