pub struct HandleEffectLayer<Next, EM>where
Next: Layer + Sync + Send + 'static,
Next::Effect: TryInto<Request<EM::Op>, Error = Next::Effect>,
EM: EffectMiddleware,{ /* private fields */ }Expand description
Middleware layer able to process some of the effects. This implements the
general behaviour making sure all follow-up effects are processed and routed
to the right place and delegates to the generic parameter EM, which
implements EffectMiddleware.
Implementations
Sourceimpl<Next, EM> HandleEffectLayer<Next, EM>
impl<Next, EM> HandleEffectLayer<Next, EM>
Sourcepub fn new(next: Next, middleware: EM) -> Self
pub fn new(next: Next, middleware: EM) -> Self
Typically, you would use Layer::handle_effects_using to construct a
HandleEffectLayer instance for a specific EffectMiddleware.